# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 OpenStack, LLC
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Tests for the functions in nova/tests/util.py
"""

import unittest

from nova import db
import nova.auth.manager as nova_auth_manager
from nova import flags
import nova.tests.utils as test_utils


# Some flags defined just for testing reset_flags()
flags.DEFINE_string('test_flag', '42', 'test flag')


FLAGS = flags.FLAGS


FLAGS.connection_type = 'fake'
FLAGS.fake_network = True


class EnvironmentResetTestCase(unittest.TestCase):

    """Tests the reset_test_environment() method"""

    def setUp(self): # pylint: disable-msg=C0103
        test_utils.reset_flags()

    def test_initial_test_environment_data(self):
        
        # The initial test environment includes 5 networks
        # with available IPs, 2 users, 4 projects. Here we
        # test that all those things have been properly
        # established.

        test_utils.reset_test_environment()
        
        num_networks = db.network_count(test_utils.ADMIN_CONTEXT)
        self.assertEquals(5, num_networks)
        
        num_users = len(db.user_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(2, num_users)

        num_projects = len(db.project_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(4, num_projects)

        # Let's add a new project for one of the user fixtures,
        # reset the test environment and re-test the number of projects
        # is still 4.

        auth_manager = nova_auth_manager.AuthManager()
        new_project = auth_manager.create_project('project 2-2', 'fakeuser2', 'project 2-2')
        num_projects = len(db.project_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(5, num_projects)

        test_utils.reset_test_environment()

        num_projects = len(db.project_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(4, num_projects)

    def test_blank_test_environment_data(self):
        
        # When called with blank=True, the test environment should
        # have absolutely no data associated with it.

        test_utils.reset_test_environment(blank=True)
        
        num_networks = db.network_count(test_utils.ADMIN_CONTEXT)
        self.assertEquals(0, num_networks)

    def test_supplied_test_environment_data(self):
        
        # When called with supplied test fixtures, the arguments should
        # override the default fixtures

        TEST_FIXTURES = {
            
            'users': 
            
            [
                ('fakeuser1', 'fakeuser1_accesskey', 'fakeuser1_secret', False),
            ],
            
            'networks':
            
            (test_utils.ADMIN_CONTEXT, FLAGS.fixed_range, 3, 16,
             FLAGS.vlan_start, FLAGS.vpn_start),

            'projects':

            [
                ('project 1-1', 'fakeuser1', 'project 1-1'),
            ],
        }

        test_utils.reset_test_environment(**TEST_FIXTURES)
        
        num_networks = db.network_count(test_utils.ADMIN_CONTEXT)
        self.assertEquals(3, num_networks)
        
        num_users = len(db.user_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(1, num_users)

        num_projects = len(db.project_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(1, num_projects)

        test_utils.reset_test_environment()
        
        num_networks = db.network_count(test_utils.ADMIN_CONTEXT)
        self.assertEquals(5, num_networks)
        
        num_users = len(db.user_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(2, num_users)

        num_projects = len(db.project_get_all(test_utils.ADMIN_CONTEXT))
        self.assertEquals(4, num_projects)
    
    def test_reset_flags_01(self):

        # Test that if we change the value of a flag in a test
        # case, that in the next test case, the flag is reset to
        # its original value
        
        self.assertEquals(FLAGS.test_flag, '42')
        
        FLAGS.test_flag = '24'

        self.assertEquals(FLAGS.test_flag, '24')

        test_utils.reset_flags()

        self.assertEquals(FLAGS.test_flag, '42')

        FLAGS.test_flag = '24'

    def test_reset_flags_02(self):

        # Test that the test_reset_flags_01 test case, which last
        # set the test_flag to 24, had that change discarded
        
        self.assertEquals(FLAGS.test_flag, '42')


if __name__ == '__main__':
    unittest.main()
