# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 OpenStack, LLC
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Utility methods used in running unit tests
"""

import logging
import os

from nova import db
from nova import context
from nova import flags
from nova import utils
import nova.auth.manager as nova_auth_manager
import nova.network.manager # for networking flags

FLAGS = flags.FLAGS

# Mapping of original flag values
_orig_flag_dict = None

# An admin context for use in setup
ADMIN_CONTEXT = context.get_admin_context()

# The set of default fixtures used in the testing environment
DEFAULT_FIXTURES = {
    
    'users': 
    
    # The test environment uses auth_manager.create_user() for
    # the creation of user fixtures, so the tuples below
    # represent the args for that function:
    #
    #   def create_user(self, name, access=None, secret=None, admin=False):
    
    [
        ('fakeuser1', 'fakeuser1_accesskey', 'fakeuser1_secret', False),
        ('fakeuser2', 'fakeuser2_accesskey', 'fakeuser2_secret', False)
    ],
    
    'networks':
    
    # The network(s) the test environment uses are created using
    # the manager from FLAGS.network_manager's create_networks()
    # method.  By default, this is VlanManager.create_networks(), which
    # has the following signature, and thus the tuple below:
    #   
    #   def create_networks(self, context, cidr, num_networks, network_size,
    #                       vlan_start, vpn_start):
    #
    # If you supply your own fixtures to reset_test_environment and have
    # altered FLAGS.network_manager, ensure the tuple you pass for your
    # network fixture matches the method signature for that network manager's
    # create_networks() method!
    
    (ADMIN_CONTEXT, FLAGS.fixed_range, 5, 16, FLAGS.vlan_start, FLAGS.vpn_start),

    'projects':

    # The test environment uses auth_manager.create_project() for
    # the creation of project fixtures, so the tuples below
    # represent the args for that function:
    #
    #   def create_project(self, name, manager_user, description=None,
    #                      member_users=None):

    [
        ('project 1-1', 'fakeuser1', 'project 1-1'),
        ('project 1-2', 'fakeuser1', 'project 1-2'),
        ('project 1-3', 'fakeuser1', 'project 1-3'),
        ('project 2-1', 'fakeuser2', 'project 2-1')
    ],
}

def reset_test_environment(blank=False, **kwargs):
    """Wipes out data in our datastore and places fresh fixture
    data into tables.

    :param blank: Start with an entirely empty test environment. Default
                  is False (start with kwargs as initial fixtures or if
                  kwargs is empty, use DEFAULT_FIXTURES)

    :param **kwargs: All keyword args are considered to be the initial
                     set of fixtures for the named argument.  For instance,
                     if reset_data() was called like so::

                         USER_FIXTURES = [
                            ('fakeuser', 'fakeaccesskey', 'fakesecret')       
                         ]

                         PROJECT_FIXTURES = [
                            ('fakeproject1', 'fakeuser', 'fake project name')
                         ]

                         reset_data(images=IMAGE_FIXTURES,
                                    instances=INSTANCE_FIXTURES)

                    then the datastore will be cleared and populated with the
                    users and project data above.


    :note If no fixtures are supplied in kwargs, then the set of default
          fixtures will be used.  See this module's DEFAULT_FIXTURES variable

    :todo This function destroys the SQLite database used in unit testing.
          If another database is used in testing, this function will not work.
          Create ability to destroy schemas in other databases used in testing.

    """

    auth_manager = nova_auth_manager.AuthManager()
    network_manager = utils.import_object(FLAGS.network_manager)
    compute_manager = utils.import_object(FLAGS.compute_manager)

    logging.debug("Purging existing data from datastore")

    # Delete the existing db and have sqlalchemy re-create
    # the database from the models.
    if FLAGS.db_backend == 'sqlalchemy':
        import nova.db.sqlalchemy.models
        nova.db.sqlalchemy.models.destroy_models()
        nova.db.sqlalchemy.models.register_models()

    if blank:
        return

    if len(kwargs) == 0:
        kwargs.update(DEFAULT_FIXTURES)

    # Generate required available networks if no fixtures
    # have been supplied.
    if 'networks' in kwargs.keys():
        logging.debug("Creating default available networks")
        network_manager.create_networks(*kwargs['networks'])

    # Generate users and projects
    if 'users' in kwargs.keys():
        logging.debug("Creating user fixtures")
        users = {}
        for fixture in kwargs['users']:
            users[fixture[0]] = auth_manager.create_user(*fixture)

        logging.debug("Creating project fixtures")
        projects = {}
        if 'projects' in kwargs.keys():
            for fixture in kwargs['projects']:
                projects[fixture[0]] = auth_manager.create_project(*fixture)
                user_context = context.RequestContext(users[fixture[1]],
                                                      projects[fixture[0]])
                network = network_manager.get_network(user_context)
                network_manager.set_network_host(ADMIN_CONTEXT,
                                                 network['id'])

    # TODO(jaypipes): Now create the instances


def reset_flags(use_test_environment=False):
    """Resets the flags to their original state
    
    :param use_test_environment: Restore the flags that were set when
                                 reset_test_environment was last called
                                 Default is False
    """
    global FLAGS, _orig_flag_dict
    
    # We do this here, as opposed to the top of the
    # module, to ensure that a test's changes to any
    # flags are kept if the test case's setUp() or
    # tearDown() methods call reset_flags()

    if not _orig_flag_dict:
        _orig_flag_dict = FLAGS.FlagValuesDict()
    
    FLAGS.Reset()
    for k, v in _orig_flag_dict.iteritems():
        setattr(FLAGS, k, v)
